# Generated by Django 4.1.7 on 2023-02-16 18:24

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("csp", "0002_cspreportblacklist"),
    ]

    operations = [
        migrations.AddField(
            model_name="csprule",
            name="created_at",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AddField(
            model_name="csprule",
            name="modified_at",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]
